import {
  Modal,
  ModalContent,
  ModalHeader,
  ModalOverlay,
  ModalProps,
  useDisclosure,
  Text,
  ModalCloseButton,
  ModalBody,
  VStack,
  FormControl,
  FormLabel,
  HStack,
  Input,
  FormErrorMessage,
  Textarea,
  ModalFooter,
  Button,
  Stack,
  Center,
  Icon,
  Badge,
  IconButton,
} from '@chakra-ui/react'
import { useFormik } from 'formik'
import React from 'react'
import { useQueryClient } from 'react-query'
import { FileWithName, Pattern } from 'src/models'
import { CreateAnimationSchema, createAnimationSchema } from './schema'
import { useDropzone } from 'react-dropzone'
import {
  MAX_FILE_SIZE,
  SIZE_TOO_LARGE,
} from 'src/pages/patterns/add-pattern/use-add-formalizations'
import { FaPaperclip, FaTrash } from 'react-icons/fa'
import prettyBytes from 'pretty-bytes'
import { CreateAnimationMutation } from '../query-builder'

const FORM_ID = 'add-animation-form'

export type AddAnimationModalProps = {
  disclosure: ReturnType<typeof useDisclosure>
  pattern: Pattern
} & Partial<ModalProps>

export const AddAnimationModal: React.FC<AddAnimationModalProps> = ({
  disclosure,
  pattern,
  ...props
}) => {
  const queryClient = useQueryClient()
  const createAnimationMutation = CreateAnimationMutation.useMutation()

  const [animationFile, setAnimationFile] = React.useState<FileWithName>(null)

  const onDrop = React.useCallback((acceptedFiles: File[]) => {
    setAnimationFile((_) => {
      const res = acceptedFiles.map(
        (f) =>
          ({
            file: f,
            name: f.name.split('.')[0] ?? f.name,
          } as FileWithName)
      )
      return res[0]
    })
  }, [])

  const dropzoneState = useDropzone({
    onDrop,
    noDrag: true,
    validator: (file) => {
      if (file.size > MAX_FILE_SIZE) {
        return {
          code: SIZE_TOO_LARGE,
          message: `Maximo ${MAX_FILE_SIZE} MB.`,
        }
      }
      return null
    },
  })

  const handleRemoveAnimationFile = () => setAnimationFile(null)

  const handleOnSubmit = async (values: CreateAnimationSchema) => {
    await createAnimationMutation.mutateAsync({
      name: values.name,
      description: values.description,
      patternId: pattern.id,
      file: animationFile.file,
    })

    disclosure.onClose()
  }

  const {
    handleSubmit,
    handleReset,
    handleChange,
    setTouched,
    values,
    isSubmitting,
    errors,
    touched,
    isValid,
  } = useFormik({
    enableReinitialize: true,
    initialValues: {
      name: '',
      description: '',
    },
    validateOnBlur: true,
    validateOnChange: true,
    validateOnMount: true,
    validationSchema: createAnimationSchema,
    onSubmit: handleOnSubmit,
  })

  const fileExtension = animationFile?.file.name.split('.')[1] ?? ''

  const isValidForm = React.useMemo(
    () => isValid && Boolean(animationFile),
    [isValid, animationFile]
  )

  return (
    <>
      {disclosure.isOpen && (
        <Modal size="xl" {...disclosure} {...props}>
          <ModalOverlay />
          <ModalContent>
            <ModalHeader>
              <Text fontSize="md">
                Crear animacion del patron {pattern.name}
              </Text>
            </ModalHeader>
            <ModalCloseButton mt="-2" mr="-2" />
            <form id={FORM_ID} onSubmit={handleSubmit} onReset={handleReset}>
              <ModalBody>
                <VStack>
                  <FormControl isInvalid={Boolean(errors.name) && touched.name}>
                    <FormLabel>
                      <HStack>
                        <Text fontSize="sm">Nombre</Text>
                        <Text color="red">*</Text>
                      </HStack>
                    </FormLabel>
                    <Input
                      name="name"
                      autoFocus
                      value={values.name}
                      onChange={(e) => {
                        setTouched({ ...touched, name: true })
                        handleChange(e)
                      }}
                    />
                    <FormErrorMessage>{errors.name}</FormErrorMessage>
                  </FormControl>
                  <FormControl
                    isInvalid={
                      Boolean(errors.description) && touched.description
                    }
                  >
                    <FormLabel>
                      <HStack>
                        <Text fontSize="sm">Descripcion</Text>
                        <Text color="red">*</Text>
                      </HStack>
                    </FormLabel>
                    <Textarea
                      name="description"
                      value={values.description}
                      onChange={(e) => {
                        setTouched({ ...touched, description: true })
                        handleChange(e)
                      }}
                    />
                    <FormErrorMessage>{errors.description}</FormErrorMessage>
                  </FormControl>

                  <Stack w="full">
                    {animationFile && (
                      <VStack w="full">
                        <Badge>{animationFile.name}</Badge>
                        <HStack
                          spacing="0"
                          w="full"
                          border="1px"
                          borderColor="gray.200"
                          borderRadius="sm"
                          bg="white"
                          width="full"
                          height="min-content"
                          mt="0"
                          minW="0"
                          flex="1"
                          alignItems="flex-start"
                          fontSize="xs"
                          h="52px"
                        >
                          <Center
                            bg="blue.600"
                            borderLeftRadius="base"
                            px="4"
                            h="52px"
                          >
                            <Text
                              fontWeight="medium"
                              variant="-1"
                              color="white"
                              textTransform="uppercase"
                            >
                              {fileExtension}
                            </Text>
                          </Center>
                          <HStack
                            h="full"
                            borderColor="gray.200"
                            borderWidth="px"
                            borderStyle="solid"
                            borderRadius="base"
                            borderLeft="none"
                            borderLeftRadius="none"
                            ml="0"
                            w="full"
                            minWidth="0"
                            flex="1"
                          >
                            <VStack
                              justifyContent={
                                animationFile.file.size
                                  ? 'center'
                                  : 'flex-start'
                              }
                              spacing="1"
                              pl="3"
                              minW="0"
                              flex="1"
                            >
                              <Text
                                variant="-1"
                                isTruncated={true}
                                w="full"
                                textAlign="center"
                              >
                                {animationFile.file.name}
                              </Text>

                              <Text variant="-2" color="gray.500">
                                {prettyBytes(animationFile.file.size ?? 0)}
                              </Text>
                            </VStack>
                            <IconButton
                              aria-label="remove animation"
                              variant="unstyled"
                              color="blue.400"
                              onClick={handleRemoveAnimationFile}
                              icon={<Icon as={FaTrash} />}
                            />
                          </HStack>
                        </HStack>
                      </VStack>
                    )}
                    {!Boolean(animationFile) && (
                      <div {...dropzoneState.getRootProps()}>
                        <Center w="100%">
                          <input
                            aria-label="file-upload"
                            {...dropzoneState.getInputProps()}
                          />
                          <HStack>
                            <Button
                              aria-label="add formalization"
                              variant="unstyled"
                              justifyContent="flex-start"
                              iconSpacing="2"
                              display="flex"
                              color="gray.500"
                              fontWeight="500"
                              fontSize="md"
                              leftIcon={<Icon as={FaPaperclip} />}
                            >
                              Agregar archivo de animacion
                            </Button>
                            <Text color="red">*</Text>
                          </HStack>
                        </Center>
                      </div>
                    )}
                  </Stack>
                </VStack>
              </ModalBody>
            </form>
            <ModalFooter>
              <HStack>
                <Button variant="outline" onClick={disclosure.onClose}>
                  Cancelar
                </Button>
                <Button
                  form={FORM_ID}
                  variant="solid"
                  colorScheme="blue"
                  type="submit"
                  isLoading={isSubmitting}
                  isDisabled={!isValidForm}
                >
                  Guardar
                </Button>
              </HStack>
            </ModalFooter>
          </ModalContent>
        </Modal>
      )}
    </>
  )
}
